#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <g711.h>
#include <audiobuffer.h>
#include <kglobal.h>
#include <kconfig.h>
#include "defines.h"
#include "rtpdataheader.h"
#include "qtimer.h"
#include "dissipate2/sipuri.h"
#include "dissipate2/sipclient.h"
#include "dspoutrtp.h"

#define BLOCKL  BLOCKL_30MS
#define NO_OF_BYTES NO_OF_BYTES_30MS

short ILBCencode( iLBC_Enc_Inst_t *iLBCenc_inst,short *encoded_data, short *data);
short ILBCdecode( iLBC_Dec_Inst_t *iLBCdec_inst,short *decoded_data, short *encoded_data, short mode);


DspOutRtp::DspOutRtp( const codecType newCodec, int newCodecNum, const QString &hostName )
{

	codec = newCodec;
	codecNum = newCodecNum;

	if( hostName != QString::null && !socket.setHostname( hostName.latin1() ) ) {
		printf( "DspOutRtp::Hostname lookup failed\n" );
	}
	dsize = 4096;  // initial max packet size
	packetbuf = new unsigned char[ sizeof( rtp_hdr_t ) + dsize ];
	bufunsend = new unsigned char[ dsize ];

	if( !( gsmInstEnc = gsm_create() ) ) {
		printf( "DspOutRtp::GSM_CREATE -error !\n" );
	}

	if( !( gsmInstDec = gsm_create() ) ) {
		printf( "DspOutRtp::GSM_CREATE -error !\n" );
	}

	outbuf = new unsigned char[ dsize ];
	tmpbuf = new unsigned char[ dsize ];
	quebuf = new unsigned char[ dsize ];
	qlen = 0;

	initEncode(&ilbcEncInst, 30 );
	initDecode(&ilbcDecInst, 30, 1);

	numunsend = 0;
	fixedrtplen = 80;
	curseq = 0;
	ts = rand();
	ssrc = 10;
	ref_sec = 0;
	ref_usec = 0;
	portnum = 0;
	audio_buf.resize( dsize * sizeof( sample_t ) );
	setDeviceName( "rtp" );
	deb_period_default=50;
	deb_period=deb_period_default;
	deb_frag=0L;
	deb_rtp=0L;

	useStun = false;
}

DspOutRtp::~DspOutRtp( void )
{
	delete packetbuf;
	delete bufunsend;
	delete outbuf;
	delete tmpbuf;
	delete quebuf;
}

bool DspOutRtp::openDevice( DeviceMode mode )
{
	devmode = mode;
	if( devmode == ReadOnly ) {
		socket.listenOnEvenPort();
		portnum = socket.getPortNumber();
		if( sendStunRequest( socket ) ) {   
			portnum = receiveStunResponse( socket );   
			if( portnum > 0 ) {
				socket.forcePortNumber( portnum );
			}
		}

//------------------------------
// Linux kernels 2.4.x
// Do not allow any double binds even when same user and REUSEADDR.
/*
		UDPMessageSocket *socketVideo = new UDPMessageSocket() ;
		socketVideo->forcePortNumber( videoPortnum );
		socketVideo->listenOnEvenPortForVideo();
		videoPortnum = socketVideo->getPortNumber();
		socketVideo->forcePortNumber( videoPortnum );
		if( sendStunRequest( *socketVideo ) ) {
			videoPortnum = receiveStunResponse( *socketVideo );
		}
		delete socketVideo;
*/
		videoPortnum = portnum + 10;
//------------------------------

	} else {
		socket.connect( portnum );
		socket.SetTOS();
	}
	KConfig *config = KGlobal::config();
	config->setGroup( "DSP" );
	deb_period_default = config->readNumEntry( "debugPeriod", 2147483647 );
	deb_period = deb_period_default;
	return true;
}

bool DspOutRtp::closeDevice( void )
{
	return true;
}

unsigned int DspOutRtp::writableFragments( void )
{
	struct timeval tv;
	int timediff;
	gettimeofday( &tv, 0 );
	timediff = ( tv.tv_sec - ref_sec ) * 1000 * 1000 + tv.tv_usec - ref_usec;
	timediff = timediff / 1000;
	ref_sec = tv.tv_sec;
	ref_usec = tv.tv_usec;
	if( timediff > 10 ) {
		return 1;
	}
	return 0;
}

bool DspOutRtp::writeBuffer( void )
{
	short *s;
	sample_t *frombuf;
	unsigned char *databuf;
	rtp_hdr_t *h = (rtp_hdr_t *) packetbuf;
	size_t fromsize;
	unsigned char *wlbuf;
	size_t bytesnew;
	size_t bytesgot;
	int tmpsize;
	unsigned int i;
	bytesnew = audio_buf.getSize() / sizeof( sample_t );
	fromsize = audio_buf.getSize() / sizeof( sample_t );
	inbuf = (unsigned char*)audio_buf.getData();
	frombuf = (sample_t *)inbuf;
	if( codec == codecGSM ) {
		sample_t *frombuf_test;
		frombuf_test = frombuf;
		s = (short *)inbuf;
		for( i=0; i<fromsize; i++) {
			s[i] = (short)(*frombuf_test);
			frombuf_test++;
		}
		tmpsize = writeGSMBuffer( gsmInstEnc, inbuf, outbuf, tmpbuf, quebuf, &qlen, fromsize );
		while( tmpsize > 0 ) {
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = 3;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += GSM_DEC_SAMPLES;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
			for( int i = 0; i < tmpsize; i++ ) {
				databuf[ i ] = outbuf[ i ];
			}
			if( socket.send( (char *) packetbuf, sizeof( rtp_hdr_t ) + tmpsize ) < 0 ) {
				return false;
			}
			tmpsize = writeGSMBuffer( gsmInstEnc, inbuf, outbuf, tmpbuf, quebuf, &qlen, 0 );
		}
	} else if( codec == codecILBC ) {
		sample_t *frombuf_test;
		frombuf_test = frombuf;
		s = (short *)inbuf;
		for( i=0; i<fromsize; i++) {
			s[i] = (short)(*frombuf_test);
			frombuf_test++;
		}
		tmpsize = writeILBCBuffer( &ilbcEncInst, inbuf, outbuf, tmpbuf, quebuf, &qlen, fromsize );
		while( tmpsize > 0 ) {
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = codecNum;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += BLOCKL;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
			for( int i = 0; i < tmpsize; i++ ) {
				databuf[ i ] = outbuf[ i ];
			}
			if( socket.send( (char *) packetbuf, sizeof( rtp_hdr_t ) + tmpsize ) < 0 ) {
				return false;
			}
			tmpsize = writeILBCBuffer( &ilbcEncInst, inbuf, outbuf, tmpbuf, quebuf, &qlen, 0 );
		}
	} else if( codec == codecULAW ) {
		wlbuf = bufunsend;
		while(bytesnew+numunsend >= fixedrtplen){
			deb_rtp++;
			deb_period--;
			h->version = 2;
			h->p = 0;
			h->x = 0;
			h->cc = 0;
			h->m = 0;
			h->pt = 0;
			h->seq = htons( curseq++ );
			h->ts = htonl( ts );
			ts += fixedrtplen;
			h->ssrc = ssrc;
			databuf = packetbuf + sizeof( rtp_hdr_t );
			bytesgot = 0;
			while( numunsend > 0 ) {
				*databuf = *wlbuf;
				databuf++;
				wlbuf++;
				numunsend--;
				bytesgot++;
			}
			wlbuf = bufunsend;
			while( bytesgot < fixedrtplen ) {
				*databuf = linear2ulaw( (short) (*frombuf) );
				databuf++;
				frombuf++;
				bytesnew--;
				bytesgot++;
			}
			if( socket.send( (char *) packetbuf, sizeof( rtp_hdr_t ) + fixedrtplen ) < 0 )
			return false;
		}

		if( bytesnew > 0 ) {
			wlbuf = bufunsend + numunsend;
			numunsend += bytesnew;
			while( bytesnew > 0 ){
				*wlbuf = linear2ulaw( (short) (*frombuf) );
				wlbuf++;
				frombuf++;
				bytesnew--;
			}
		}
		deb_frag++;
		if(deb_period <= 0){
			deb_period = deb_period_default;
			printf("audio: %10lu frag-%d, %10lu rtp-%d, %5d unsent\n",
				deb_frag, fromsize, deb_rtp, fixedrtplen, numunsend);
			fflush(stdout);
		}
	}

	return true;
}

bool DspOutRtp::setSampleRate( int newrate )
{
	if( newrate != 8000 ) return false;
	rate = newrate;
	return true;
}

bool DspOutRtp::setPortNum( int newport )
{
	portnum = newport;
	return true;
}

unsigned int DspOutRtp::readableFragments( void )
{
	struct timeval timeout;
	fd_set read_fds;
	int highest_fd;
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	FD_ZERO( &read_fds );
	FD_SET( socket.getFileDescriptor(), &read_fds );
	highest_fd = socket.getFileDescriptor() + 1;
retry:
	if ( select( highest_fd, &read_fds, NULL, NULL, &timeout ) == -1 ) {
		if ( errno == EINTR ) goto retry;
		perror( "DspOutRtp::doSelect(): select() punted" );
		exit( 1 );
	}
	if ( FD_ISSET( socket.getFileDescriptor(), &read_fds ) ) {
		return 1;
	}
	return 0;
}

bool DspOutRtp::readBuffer( void )
{
	unsigned char *inbuf;
	sample_t *outbuf;
	int recvsize;
	int i;
	int size;
	recvsize = socket.receive( (char *) packetbuf, sizeof( rtp_hdr_t ) + dsize );
	rtp_hdr_t *h = (rtp_hdr_t *) packetbuf;
	if( h->pt == 0 ) {
		audio_buf.resize( ( recvsize - (int) sizeof( rtp_hdr_t ) ) * sizeof( sample_t ) );
		outbuf = (sample_t *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
		for( i = 0; i < recvsize - (int) sizeof( rtp_hdr_t ); i++ ) {
			*outbuf++ = (sample_t) ulaw2linear( inbuf[ i ] );
		}
	} else if( h->pt == 3 ) {
		audio_buf.resize( GSM_DEC_SAMPLES * sizeof( sample_t ) );
		outbuf = (sample_t *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
		readGSMBuffer( gsmInstDec, inbuf, tmpbuf, false );
		short *s = (short *)tmpbuf;
		for( i = 0; i < GSM_DEC_SAMPLES; i++) {
			*outbuf++ = (sample_t)(s[i]);
		}
	} else if( codec == codecILBC && (int)h->pt == codecNum ) {
		audio_buf.resize( BLOCKL * sizeof( sample_t ) );
		outbuf = (sample_t *) audio_buf.getData();
		inbuf = &packetbuf[ sizeof( rtp_hdr_t ) ];
		size = readILBCBuffer( &ilbcDecInst, inbuf, tmpbuf, 1, false );
		short *s = (short *)tmpbuf;
		for( i = 0; i < size; i++) {
			*outbuf++ = (sample_t)(s[i]);
		}
	}
	return true;
}

bool DspOutRtp::sendStunRequest( UDPMessageSocket &socketStun )
{
	SipUri stun;
	if( useStun ) {
		stun = SipUri( stunSrv );
		if( !socketStun.setHostname( stun.getHostname() ) ) {
			return false;
		}
		socketStun.connect( stun.getPortNumber() );
		printf( "DspOutRtp: STUN request\n" );
		StunRequestSimple req;
		req.msgHdr.msgType = htons(BindRequestMsg);
		req.msgHdr.msgLength = htons( sizeof(StunRequestSimple)-sizeof(StunMsgHdr) );
		for ( int i=0; i<16; i++ ) {
			req.msgHdr.id.octet[i]=0;
		}
		int id = rand();
		req.msgHdr.id.octet[0] = id;
		req.msgHdr.id.octet[1] = id>>8;
		req.msgHdr.id.octet[2] = id>>16;
		req.msgHdr.id.octet[3] = id>>24;
		socketStun.send( (char *)&req, sizeof( req ) );
	} else {
		return false;
	}
	return true;
}

unsigned int DspOutRtp::receiveStunResponse( UDPMessageSocket &socketStun )
{
	unsigned int port = 0;
	char inputbuf[ 8000 ];
	int bytesread;
	struct timeval timeout;
	fd_set read_fds;
	int highest_fd;
	timeout.tv_sec = 5;
	timeout.tv_usec = 0;
	FD_ZERO( &read_fds );
	FD_SET( socketStun.getFileDescriptor(), &read_fds );
	highest_fd = socketStun.getFileDescriptor() + 1;
retry:
	if( select( highest_fd, &read_fds, NULL, NULL, &timeout ) == -1 ) {
		if( errno == EINTR ) goto retry;
		perror( "SipClient::doSelect(): select() punted" );
		exit( 1 );
	}
	if( FD_ISSET( socketStun.getFileDescriptor(), &read_fds ) ) {
		printf( "SipClient: Receiving message...\n" );
		bytesread = read( socketStun.getFileDescriptor(), inputbuf, 8000 - 1 );
		StunMsgHdr* hdr = reinterpret_cast<StunMsgHdr*>( inputbuf );
		if( hdr->msgType == BindResponseMsg ) {
			printf( "SipClient: STUN response for RTP\n" );
			char* body = inputbuf + sizeof( StunMsgHdr );
			unsigned int size = ntohs( hdr->msgLength );
			while( size > 0 ) {
				StunAtrHdr* attr = reinterpret_cast<StunAtrHdr*>( body );
				unsigned int hdrLen = ntohs( attr->length );
				if( ntohs( attr->type ) == MappedAddress ) {
					StunAtrAddress* attribute = reinterpret_cast<StunAtrAddress*>( body );
					if ( attribute->address.addrHdr.family == IPv4Family ) {
						StunAtrAddress4* atrAdd4 = reinterpret_cast<StunAtrAddress4*>( body );
						if ( hdrLen == sizeof( StunAtrAddress4 ) - 4 ) {
							port = ntohs( atrAdd4->addrHdr.port );
						}
					}
				}
				body += hdrLen+4;
				size -= hdrLen+4;
			}
		}
	}
	return port;
}


/**********************************************************************************/
/* GSM                                                                            */
/**********************************************************************************/

int DspOutRtp::writeGSMBuffer( gsm gsmInst,
		unsigned char *input_buf, unsigned char *output_buf,
		unsigned char *tmp_buf, unsigned char *queue,
		int *qlen, int size )
{
	int outbufsize = 0;
	int inq, inx;
	short *ibuf;
	short *obuf;
	short *tbuf;
	short *qbuf;
	ibuf = (short *)input_buf;
	obuf = (short *)output_buf;
	qbuf = (short *)queue;
	inq = *qlen;
	inx = 0;
	if( inq >= GSM_DEC_SAMPLES ) {
		tbuf = (short *)tmp_buf;
		while( inx < GSM_DEC_SAMPLES ) {
			*tbuf = *qbuf;
			tbuf++;
			qbuf++;
			inq--;
			inx++;
		}
		tbuf = (short *)tmp_buf;
		gsm_encode( gsmInst, (gsm_signal *)tbuf, (gsm_byte *)obuf);
		*qlen = inq;
		outbufsize = GSM_ENC_BYTES;
	} else if( inq + size >= GSM_DEC_SAMPLES ) {
		tbuf = (short *)tmp_buf;
		while( inq > 0 ) {
			*tbuf = *qbuf;
			tbuf++;
			qbuf++;
			inq--;
			inx++;
		}
		*qlen = 0;
		while( inx < GSM_DEC_SAMPLES ) {
			*tbuf = *ibuf;
			tbuf++;
			ibuf++;
			size--;
			inx++;
		}
		tbuf = (short *)tmp_buf;
		gsm_encode( gsmInst, (gsm_signal *)tbuf, (gsm_byte *)obuf);
		outbufsize = GSM_ENC_BYTES;

	}
	if( size > 0 ) {
		inq = *qlen;
		qbuf = (short *)(queue + ( inq * sizeof(short) ) );
		while( size > 0 ) {
			*qbuf = *ibuf;
			inq++;
			size--;
			qbuf++;
			ibuf++;
		}
		*qlen = inq;
	}
	return outbufsize;
}

int DspOutRtp::readGSMBuffer( gsm Gsm_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		int ignore )
{
	short *outbuf;
	short *ibuf;
	outbuf = (short *)output_buf;
	ibuf = (short *)input_buf;
	if( !ignore ) {
		gsm_decode( Gsm_Inst, (gsm_byte *)ibuf, (gsm_signal *)outbuf );
	}
	return true;
}



/**********************************************************************************/
/* iLBC                                                                           */
/**********************************************************************************/

int DspOutRtp::writeILBCBuffer( iLBC_Enc_Inst_t *Enc_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		unsigned char *tmp_buf, unsigned char *queue,
		int *qlen, int size )
{
	int outbufsize = 0;
	int inq, inx;
	short *ibuf;
	short *obuf;
	short *tbuf;
	short *qbuf;
	ibuf = (short *)input_buf;
	obuf = (short *)output_buf;
	qbuf = (short *)queue;
	inq = *qlen;
	inx = 0;
	if( inq >= BLOCKL ) {
		tbuf = (short *)tmp_buf;
		while( inx < BLOCKL ) {
			*tbuf = *qbuf;
			tbuf++;
			qbuf++;
			inq--;
			inx++;
		}
		tbuf = (short *)tmp_buf;
		outbufsize = ILBCencode( Enc_Inst, obuf, tbuf);
		*qlen = inq;
	} else if( inq + size >= BLOCKL ) {
		tbuf = (short *)tmp_buf;
		while( inq > 0 ) {
			*tbuf = *qbuf;
			tbuf++;
			qbuf++;
			inq--;
			inx++;
		}
		*qlen = 0;
		while( inx < BLOCKL ) {
			*tbuf = *ibuf;
			tbuf++;
			ibuf++;
			size--;
			inx++;
		}
		tbuf = (short *)tmp_buf;
		outbufsize = ILBCencode( Enc_Inst, obuf, tbuf);
	}
	if( size > 0 ) {
		inq = *qlen;
		qbuf = (short *)(queue + ( inq * sizeof(short) ) );
		while( size > 0 ) {
			*qbuf = *ibuf;
			inq++;
			size--;
			qbuf++;
			ibuf++;
		}
		*qlen = inq;
	}
	return outbufsize;
}


int DspOutRtp::readILBCBuffer( iLBC_Dec_Inst_t *Dec_Inst,
		unsigned char *input_buf, unsigned char *output_buf,
		short mode, int ignore)
{
	short *outbuf;
	short *ibuf;
	outbuf = (short *)output_buf;
	ibuf = (short *)input_buf;
	if( !ignore ) {
		return ILBCdecode( Dec_Inst, outbuf, ibuf, mode );
	}
	return 0;
}


void  DspOutRtp::setStunSrv( QString newStunSrv )
{
	useStun = true;
	stunSrv = newStunSrv;
}

/*----------------------------------------------------------------*
 *  Encoder interface function
 *---------------------------------------------------------------*/

short ILBCencode(iLBC_Enc_Inst_t *iLBCenc_inst,short *encoded_data,short *data)
{
	float block[BLOCKL];
	int k;

/* convert signal to float */

	for(k=0;k<BLOCKL;k++) block[k] = (float)data[k];

/* do the actual encoding */

	iLBC_encode((unsigned char *)encoded_data,block,iLBCenc_inst);

	return (short)(NO_OF_BYTES);
}


/*----------------------------------------------------------------*
 *  Decoder interface function
 *---------------------------------------------------------------*/

short ILBCdecode(iLBC_Dec_Inst_t *iLBCdec_inst,
		short *decoded_data, short *encoded_data, short mode)
{
	int k;
	float decblock[BLOCKL], dtmp;

/* check if mode is valid */

	if(mode<0 || mode>1){
		printf("\nERROR - Wrong mode - 0, 1 allowed\n");
		exit(3);
	}

/* do actual decoding of block */

	iLBC_decode(decblock, (unsigned char *)encoded_data, iLBCdec_inst, mode);

/* convert to short */

	for(k=0;k<BLOCKL;k++){
		dtmp=decblock[k];
		if (dtmp<MIN_SAMPLE)dtmp=MIN_SAMPLE;
		else if (dtmp>MAX_SAMPLE)dtmp=MAX_SAMPLE;
		decoded_data[k] = (short) dtmp;
	}

	return (short)BLOCKL;
}

